@@ -14,7 +14,7 @@
  * Maintainer: Jörg Behrens <behrens@dkrz.de>
  *             Moritz Hanke <hanke@dkrz.de>
  *             Thomas Jahns <jahns@dkrz.de>
- * URL: https://doc.redmine.dkrz.de/yaxt/html/
+ * URL: https://dkrz-sw.gitlab-pages.dkrz.de/yaxt/
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are  permitted provided that the following conditions are
@@ -57,14 +57,16 @@
 #define VERBOSE
 
 #include "tests.h"
+#include "ctest_common.h"
 #include "test_idxlist_utils.h"
 #include "core/ppm_xfuncs.h"
 
-int main(void) {
+static void
+test_idxsections(void);
 
-  // init mpi
+int main(int argc, char **argv) {
 
-  xt_mpi_call(MPI_Init(NULL, NULL), MPI_COMM_WORLD);
+  test_init_mpi(&argc, &argv, MPI_COMM_WORLD);
 
   xt_initialize(MPI_COMM_WORLD);
 
@@ -261,9 +263,12 @@ int main(void) {
     xt_idxlist_get_bounding_box(collectionlist, ndim, global_size_bb,
                                 global_start_index, bounds);
 
+    bool mismatch = false;
     for (size_t i = 0; i < ndim; ++i)
-      if (bounds[i].size != 0)
-        PUT_ERR("ERROR: xt_idxlist_get_bounding_box\n");
+      mismatch |= (bounds[i].size != 0);
+
+    if (mismatch)
+      PUT_ERR("ERROR: xt_idxlist_get_bounding_box\n");
 
     xt_idxlist_delete(collectionlist);
   }
@@ -299,19 +304,109 @@ int main(void) {
 
     static const Xt_int ref_start[3] = { 2, 2, 1 };
 
+    bool mismatch = false;
     for (size_t i = 0; i < ndim; ++i)
-      if (bounds[i].size != 2 || bounds[i].start != ref_start[i])
-        PUT_ERR("ERROR: xt_idxlist_get_bounding_box\n");
+      mismatch |= (bounds[i].size != 2 || bounds[i].start != ref_start[i]);
+
+    if (mismatch)
+      PUT_ERR("ERROR: xt_idxlist_get_bounding_box\n");
 
     xt_idxlist_delete(collectionlist);
   }
 
+  test_idxsections();
+
   xt_finalize();
   xt_mpi_call(MPI_Finalize(), MPI_COMM_WORLD);
 
   return TEST_EXIT_CODE;
 }
 
+static void
+test_idxsections(void)
+{
+  enum { irank = 4, num_regions=5, global_index_start=-1 };
+
+  /* global size */
+  static const Xt_int gsz[irank]
+    = { 1, 1, 122, 132 };
+  static const int gsz_i[irank]
+    = { 1, 1, 122, 132 };
+  /* slice size */
+  static const int lsz[num_regions][irank]
+    = { { 1, 1, 15,  8 }, // core region
+        { 1, 1,  3, 14 }, // 1st halo
+        { 1, 1,  3, 14 }, // 2nd halo
+        { 1, 1, 15,  3 }, // 3rd halo
+        { 1, 1, 15,  3 } }; // 4th halo
+
+  /* slice start coordinate */
+  static const Xt_int lst[num_regions][irank]
+    = { { 0, 0, 46, 42 },
+        { 0, 0, 43, 39 },
+        { 0, 0, 61, 39 },
+        { 0, 0, 46, 50 },
+        { 0, 0, 46, 39 } };
+
+  Xt_idxlist component_idxlists[num_regions];
+
+  size_t num_indices = 0;
+  for (size_t i = 0; i < num_regions; ++i) {
+    size_t product = 1;
+    for (size_t j = 0; j < irank; ++j)
+      product *= (size_t)lsz[i][j];
+    num_indices += product;
+  }
+  assert(num_indices < INT_MAX);
+  Xt_int *restrict indices = xmalloc(sizeof (*indices) * num_indices);
+
+  size_t idx_ofs = 0;
+
+  /* sub-slices or regions */
+  for (size_t i = 0; i < num_regions; ++i) {
+    component_idxlists[i] = xt_idxsection_new(
+      global_index_start, irank, gsz, lsz[i], lst[i]);
+
+    Xt_int idx_stride4 = (Xt_int)(gsz[3] * gsz[2] * gsz[0]),
+      idx_stride3 = (Xt_int)(gsz[3] * gsz[2]),
+      idx_stride2 = gsz[3];
+    for (size_t i4 = 0; i4 < (size_t)lsz[i][0]; ++i4) {
+      Xt_int dim_ofs4 = (Xt_int)((lst[i][0]+(Xt_int)i4) * idx_stride4);
+      for (size_t i3 = 0; i3 < (size_t)lsz[i][1]; ++i3) {
+        Xt_int dim_ofs3
+          = (Xt_int)((lst[i][1]+(Xt_int)i3) * idx_stride3 + dim_ofs4);
+        for (size_t i2 = 0; i2 < (size_t)lsz[i][2]; ++i2) {
+          Xt_int dim_ofs2
+            = (Xt_int)((lst[i][2]+(Xt_int)i2) * idx_stride2 + dim_ofs3);
+          for (size_t i1 = 0; i1 < (size_t)lsz[i][3]; ++i1)
+            indices[idx_ofs+i1] = (Xt_int)
+              (lst[i][3] + (Xt_int)i1 + global_index_start + dim_ofs2);
+          idx_ofs += (size_t)lsz[i][3];
+        }
+      }
+    }
+  }
+
+  Xt_idxlist collection_idxlist
+    = xt_idxlist_collection_new(component_idxlists, num_regions);
+  check_idxlist(collection_idxlist, indices, (int)num_indices);
+
+  // global view
+  Xt_idxlist global_section = xt_idxsection_new(
+    global_index_start, irank, gsz, gsz_i, (Xt_int [irank]){ 0, 0, 0, 0 });
+
+  Xt_idxlist intersection
+    = xt_idxlist_get_intersection(global_section, collection_idxlist);
+  xt_sort_xt_int(indices, num_indices);
+  check_idxlist(intersection, indices, (int)num_indices);
+  xt_idxlist_delete(intersection);
+  xt_idxlist_delete(global_section);
+  xt_idxlist_delete(collection_idxlist);
+  for (size_t i = 0; i < num_regions; ++i)
+    xt_idxlist_delete(component_idxlists[i]);
+  free(indices);
+}
+
 /*
  * Local Variables:
  * c-basic-offset: 2
